@@ -46,13 +46,16 @@
 #include <sys/types.h>
 #endif /* defined(HAVE_SYS_TYPES_H) && defined(HAVE_SYS_STAT_H) */
 
+#if defined(WIN32) || defined(_WIN32)
+#include <io.h>
+#endif
+
 /*
  * Optional library and BFT headers
  */
 
 #include "bft_error.h"
 #include "bft_file.h"
-#include "bft_intl.h"
 #include "bft_mem.h"
 
 /*-----------------------------------------------------------------------------*/
@@ -480,7 +483,7 @@ _bft_file_read(void              *const rec,
   /* Check file state */
 
   assert(f != NULL);
-  assert(rec != NULL);
+  assert(rec != NULL || ni == 0);
   assert(   f->type == BFT_FILE_TYPE_BINARY
          || f->type == BFT_FILE_TYPE_FORTRAN_BINARY);
   assert(f->mode == BFT_FILE_MODE_READ);
@@ -865,7 +868,7 @@ bft_file_eof(const bft_file_t  *const f)
 int
 bft_file_flush(bft_file_t  *const f)
 {
-  int retval;
+  int retval = 0;
 
   assert(f != NULL);
 
@@ -1463,7 +1466,7 @@ bft_file_write(const void        *const rec,
   /* Check file state */
 
   assert(f != NULL);
-  assert(rec != NULL);
+  assert(rec != NULL || ni == 0);
   assert(   f->type == BFT_FILE_TYPE_BINARY
          || f->type == BFT_FILE_TYPE_FORTRAN_BINARY);
   assert(f->mode == BFT_FILE_MODE_APPEND || f->mode == BFT_FILE_MODE_WRITE);
@@ -1621,6 +1624,13 @@ bft_file_mkdir_default(const char  *const pathname)
 
 #if defined(HAVE_MKDIR)
 
+#if defined(WIN32) || defined(_WIN32)
+
+  mkdir(pathname);
+  return 0;
+
+#else
+
   if (mkdir(pathname, S_IRWXU|S_IRWXG|S_IRWXO) != 0) {
 
     if (errno == EEXIST) {
@@ -1659,6 +1669,8 @@ bft_file_mkdir_default(const char  *const pathname)
 
   } /* End of directory creation failure case */
 
+#endif
+
   return 0;
 
 #else /* #if defined(HAVE_MKDIR) */
