@@ -43,7 +43,6 @@
  */
 
 #include "bft_error.h"
-#include "bft_intl.h"
 #include "bft_mem_usage.h"
 #include "bft_mem.h"
 #include "bft_printf.h"
@@ -252,10 +251,7 @@ static void _bft_mem_summary(FILE  *f)
 {
   char unit;
   unsigned long value[2];
-  int track_mode;
   size_t mem_usage;
-  static const char *track_state[2] = {N_("off"),
-                                       N_("on")};
 
   if (f == NULL)
     return;
@@ -287,8 +283,6 @@ static void _bft_mem_summary(FILE  *f)
 
   if (bft_mem_usage_initialized() == 1) {
 
-    bft_mem_usage_get_options(&track_mode);
-
     /* Maximum measured memory */
 
     mem_usage = bft_mem_usage_max_pr_size();
@@ -296,32 +290,6 @@ static void _bft_mem_summary(FILE  *f)
       fprintf(f,
               _("Maximum program memory measure:  %8lu kB\n"),
               (unsigned long)mem_usage);
-      fprintf(f,
-              _("  program memory tracking: %s\n"),
-              _(track_state[(  track_mode
-                             & BFT_MEM_USAGE_TRACK_PR_SIZE) ? 1 : 0]));
-    }
-
-    mem_usage = bft_mem_usage_max_alloc_size();
-    if (mem_usage > 0) {
-      fprintf(f,
-              _("Maximum malloc() memory measure: %8lu kB\n"),
-              (unsigned long)mem_usage);
-      fprintf(f,
-              _("  malloc() memory tracking: %s\n"),
-              _(track_state[(  track_mode
-                             & BFT_MEM_USAGE_TRACK_ALLOC_SIZE) ? 1 : 0]));
-    }
-
-    mem_usage = bft_mem_usage_max_heap_size();
-    if (mem_usage > 0) {
-      fprintf(f,
-              _("Maximum heap size measure:       %8lu kB\n"),
-              (unsigned long)mem_usage);
-      fprintf(f,
-              _("  heap size tracking: %s\n"),
-              _(track_state[(  track_mode
-                             & BFT_MEM_USAGE_TRACK_HEAP_SIZE) ? 1 : 0]));
     }
 
     /* Current measured memory */
@@ -329,21 +297,8 @@ static void _bft_mem_summary(FILE  *f)
     mem_usage = bft_mem_usage_pr_size();
     if (mem_usage > 0)
       fprintf(f,
-              _("Current program memory measure:  %8lu kB\n"),
+              _("Current program memory measure:   %8lu kB\n"),
               (unsigned long)mem_usage);
-
-    mem_usage = bft_mem_usage_alloc_size();
-    if (mem_usage > 0)
-      fprintf(f,
-              _("Current malloc() memory measure: %8lu kB\n"),
-              (unsigned long)mem_usage);
-
-    mem_usage = bft_mem_usage_heap_size();
-    if (mem_usage > 0)
-      fprintf(f,
-              _("Current heap size measure:       %8lu kB\n"),
-              (unsigned long)mem_usage);
-
   }
 
 }
@@ -1065,6 +1020,8 @@ bft_mem_memalign(size_t       alignment,
   _bft_mem_error(file_name, line_num, errno,
                  _("No aligned allocation function available on this system"));
 
+  return NULL;
+
 #endif
 }
 
