@@ -6,7 +6,7 @@
   This file is part of the "Base Functions and Types" library, intended to
   simplify and enhance portability, memory and I/O use for scientific codes.
 
-  Copyright (C) 2004-2008  EDF
+  Copyright (C) 2004-2009  EDF
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -127,12 +127,10 @@ extern "C" {
 static int  _bft_mem_usage_global_initialized = 0;
 
 static size_t _bft_mem_usage_global_max_pr = 0;
-static size_t _bft_mem_usage_global_max_alloc = 0;
-static size_t _bft_mem_usage_global_max_heap = 0;
 
-#if defined(HAVE_SBRK)
+#if defined(USE_SBRK)
 static void  *_bft_mem_usage_global_init_sbrk = NULL;
-#endif /* (HAVE_SBRK) */
+#endif
 
 #if defined (bft_OS_Linux) && defined(HAVE_SYS_STAT_H) \
                            && defined(HAVE_SYS_TYPES_H)
@@ -156,9 +154,6 @@ static void
 
 static int _bft_mem_usage_global_use_hooks = 0;
 static int _bft_mem_usage_global_up = 0;
-static int _bft_mem_usage_global_track_mode = (  BFT_MEM_USAGE_TRACK_PR_SIZE
-                                               | BFT_MEM_USAGE_TRACK_ALLOC_SIZE
-                                               | BFT_MEM_USAGE_TRACK_HEAP_SIZE);
 
 #endif /* (HAVE_MALLOC_HOOKS) */
 
@@ -168,144 +163,6 @@ static int _bft_mem_usage_global_track_mode = (  BFT_MEM_USAGE_TRACK_PR_SIZE
 
 #if defined(HAVE_MALLOC_HOOKS)
 
-/*!
- * \brief Limit tracking mode so as to avoid unneded operations.
- */
-
-static void
-_limit_track_mode(void)
-{
-#if defined (bft_OS_Linux) && defined(HAVE_SYS_STAT_H) \
-                           && defined(HAVE_SYS_TYPES_H)
-  if (   _bft_mem_usage_proc_file_init == 0
-      || _bft_mem_usage_proc_file_peak == 1)
-    _bft_mem_usage_global_track_mode = (  _bft_mem_usage_global_track_mode
-                                        & (  BFT_MEM_USAGE_TRACK_ALLOC_SIZE
-                                           | BFT_MEM_USAGE_TRACK_HEAP_SIZE));
-#endif
-}
-
-#endif /* (HAVE_MALLOC_HOOKS) */
-
-#if defined (bft_OS_Linux) && defined(HAVE_SYS_STAT_H) \
-                           && defined(HAVE_SYS_TYPES_H)
-
-/*!
- * \brief Initialize current process memory use count depending on system.
- */
-
-static void
-_bft_mem_usage_pr_size_init(void)
-{
-  char  buf[512]; /* should be large enough for "/proc/%lu/status"
-                     then beginning of file content */
-  size_t  r_size, i;
-  _Bool   status_has_peak;
-  const pid_t  pid = getpid();
-
-  /*
-    Under Linux with procfs, one line of the pseudo-file "/proc/pid/status"
-    (where pid is the process number) is of the following form:
-    VmSize:     xxxx kB
-    This line may be the 10th for a 2.4 kernel, or the 12th to 13th for
-    a 2.6.x kernel. On more recent 2.6.x kernels, another line (the 12th)
-    is of the form:
-    VmPeak:     xxxx kB
-    When VmSize is available but not VmPeak, we use low-level file I/O on
-    this file (using read() instead of fread()) to avoid update issues due
-    to buffering when keeping the file open and re-reading the area
-    containing the "VmSize" information.
-    When VmPeak is also available, memory use tracking is not necessary, so
-    the number of calls to bft_mem_usage_pr_size() is presumably much lower
-    (being based solely on the user code, not on bft_mem_usage's tracking),
-    so we prefer to open and close "/proc/pid/status" on each
-    call to bft_mem_usage_pr_size().
-  */
-
-  if (_bft_mem_usage_proc_file_init != 0)
-    return;
-
-  sprintf(buf, "/proc/%lu/status", (unsigned long) pid);
-
-  _bft_mem_usage_proc_file_fd = open(buf, O_RDONLY);
-
-  if (_bft_mem_usage_proc_file_fd != -1) {
-
-    r_size = read(_bft_mem_usage_proc_file_fd, buf, 512);
-
-    if (r_size > 32) { /* Leave a margin for "VmPeak" or "VmSize:" line */
-      r_size -= 32;
-      for (i = 0; i < r_size; i++) {
-        if (buf[i] == 'V' && strncmp(buf+i, "VmPeak:", 7) == 0) {
-          status_has_peak = true;
-          break;
-        }
-      }
-      for (i = 0; i < r_size; i++) {
-        if (buf[i] == 'V' && strncmp(buf+i, "VmSize:", 7) == 0)
-          break;
-      }
-      /* If VmSize was found, proc file may be used */
-      if (i < r_size) {
-        if (status_has_peak == true) {
-          _bft_mem_usage_proc_file_peak = 1;
-        }
-        else {
-          /* set search position, leaving small margin ahead of position
-             in case file content length before "VmSize:" changes
-             (typically by 1 or 2 chars) */
-          i = (i > 16) ? i -16 : 0;
-          _bft_mem_usage_proc_file_pos = i;
-        }
-        _bft_mem_usage_proc_file_init = 1;
-      }
-    }
-
-    /* Close file if unusable or if VmPeak is available */
-    if (   _bft_mem_usage_proc_file_init == -1
-        || _bft_mem_usage_proc_file_peak == 1) {
-      (void)close(_bft_mem_usage_proc_file_fd);
-      _bft_mem_usage_proc_file_fd = -1;
-    }
-  }
-
-  /* If initialization failed for some reason (proc file unavailable or does
-     or does not contain the required fields), mark method as unusable */
-  if (_bft_mem_usage_proc_file_init == 0)
-    _bft_mem_usage_proc_file_init = -1;
-
-#if defined(HAVE_MALLOC_HOOKS)
-  /* Disable BFT_MEM_USAGE_TRACK_PR_SIZE if it is unneeded */
-  _limit_track_mode();
-#endif
-}
-
-/*!
- * \brief Finalize current process memory use count depending on system.
- */
-
-static void
-_bft_mem_usage_pr_size_end(void)
-{
-  if (_bft_mem_usage_proc_file_init != 1)
-    return;
-
-  if (_bft_mem_usage_proc_file_fd != -1) {
-    (void)close(_bft_mem_usage_proc_file_fd);
-    _bft_mem_usage_proc_file_init = 0;
-    _bft_mem_usage_proc_file_fd = -1;
-  }
-}
-
-#else  /* defined (bft_OS_Linux) && ... */
-
-#define _bft_mem_usage_pr_size_init()
-#define _bft_mem_usage_pr_size_end()
-
-#endif /* defined (bft_OS_Linux) && ... */
-
-#if defined(HAVE_MALLOC_HOOKS)
-
 /*
  * Memory size update to be called by malloc_hook functions.
  */
@@ -313,14 +170,7 @@ _bft_mem_usage_pr_size_end(void)
 static void
 _bft_mem_usage_size_update(void)
 {
-  if (_bft_mem_usage_global_track_mode & BFT_MEM_USAGE_TRACK_PR_SIZE)
-    (void) bft_mem_usage_pr_size();
-
-  if (_bft_mem_usage_global_track_mode & BFT_MEM_USAGE_TRACK_ALLOC_SIZE)
-    (void) bft_mem_usage_alloc_size();
-
-  if (_bft_mem_usage_global_track_mode & BFT_MEM_USAGE_TRACK_HEAP_SIZE)
-    (void) bft_mem_usage_heap_size();
+  (void) bft_mem_usage_pr_size();
 }
 
 /*
@@ -334,7 +184,7 @@ _bft_mem_usage_size_update(void)
  * Electric Fence.
  *
  * returns:
- *   NULL Pointer.
+ *   1 (NULL preferred, 1 should avoid TotalView warning).
  */
 
 static __malloc_ptr_t
@@ -342,7 +192,7 @@ _bft_mem_usage_malloc_hook_test(size_t size,
                                 const __malloc_ptr_t _ptr)
 {
   _bft_mem_usage_global_use_hooks = 1;
-  return NULL;
+  return (__malloc_ptr_t)1;
 }
 
 /*
@@ -460,8 +310,8 @@ _bft_mem_usage_set_hooks(void)
     old_malloc_hook = __malloc_hook;
     __malloc_hook = _bft_mem_usage_malloc_hook_test;
     ptr_test = malloc(128);
-    /* We may have a non NULL value if hooks are not active */
-    if (ptr_test != NULL)
+    /* We have really allocated ptr_test if hooks are not active */
+    if (ptr_test != NULL && _bft_mem_usage_global_use_hooks == 0)
       free(ptr_test);
     __malloc_hook = old_malloc_hook;
 
@@ -485,11 +335,6 @@ _bft_mem_usage_set_hooks(void)
     }
 
   }
-
-  /* If hooks are not available, set all tracking options to zero */
-
-  else
-    _bft_mem_usage_global_track_mode = 0;
 }
 
 /*
@@ -500,14 +345,132 @@ static void
 _bft_mem_usage_unset_hooks(void)
 {
   if (_bft_mem_usage_global_use_hooks != 0) {
-     __malloc_hook  = _bft_mem_usage_old_malloc_hook;
-     __realloc_hook = _bft_mem_usage_old_realloc_hook;
-     __free_hook    = _bft_mem_usage_old_free_hook;
+    __malloc_hook  = _bft_mem_usage_old_malloc_hook;
+    __realloc_hook = _bft_mem_usage_old_realloc_hook;
+    __free_hook    = _bft_mem_usage_old_free_hook;
+    _bft_mem_usage_global_use_hooks = 0;
   }
 }
 
 #endif /* defined(HAVE_MALLOC_HOOKS) */
 
+#if defined (bft_OS_Linux) && defined(HAVE_SYS_STAT_H) \
+                           && defined(HAVE_SYS_TYPES_H)
+
+/*!
+ * \brief Initialize current process memory use count depending on system.
+ */
+
+static void
+_bft_mem_usage_pr_size_init(void)
+{
+  char  buf[512]; /* should be large enough for "/proc/%lu/status"
+                     then beginning of file content */
+  size_t  r_size, i;
+  _Bool   status_has_peak = false;
+  const pid_t  pid = getpid();
+
+  /*
+    Under Linux with procfs, one line of the pseudo-file "/proc/pid/status"
+    (where pid is the process number) is of the following form:
+    VmSize:     xxxx kB
+    This line may be the 10th for a 2.4 kernel, or the 12th to 13th for
+    a 2.6.x kernel. On more recent 2.6.x kernels, another line (the 12th)
+    is of the form:
+    VmPeak:     xxxx kB
+    When VmSize is available but not VmPeak, we use low-level file I/O on
+    this file (using read() instead of fread()) to avoid update issues due
+    to buffering when keeping the file open and re-reading the area
+    containing the "VmSize" information.
+    When VmPeak is also available, memory use tracking is not necessary, so
+    the number of calls to bft_mem_usage_pr_size() is presumably much lower
+    (being based solely on the user code, not on bft_mem_usage's tracking),
+    so we prefer to open and close "/proc/pid/status" on each
+    call to bft_mem_usage_pr_size().
+  */
+
+  if (_bft_mem_usage_proc_file_init != 0)
+    return;
+
+  sprintf(buf, "/proc/%lu/status", (unsigned long) pid);
+
+  _bft_mem_usage_proc_file_fd = open(buf, O_RDONLY);
+
+  if (_bft_mem_usage_proc_file_fd != -1) {
+
+    r_size = read(_bft_mem_usage_proc_file_fd, buf, 512);
+
+    if (r_size > 32) { /* Leave a margin for "VmPeak" or "VmSize:" line */
+      r_size -= 32;
+      for (i = 0; i < r_size; i++) {
+        if (buf[i] == 'V' && strncmp(buf+i, "VmPeak:", 7) == 0) {
+          status_has_peak = true;
+          break;
+        }
+      }
+      for (i = 0; i < r_size; i++) {
+        if (buf[i] == 'V' && strncmp(buf+i, "VmSize:", 7) == 0)
+          break;
+      }
+      /* If VmSize was found, proc file may be used */
+      if (i < r_size) {
+        if (status_has_peak == true) {
+          _bft_mem_usage_proc_file_peak = 1;
+        }
+        else {
+          /* set search position, leaving small margin ahead of position
+             in case file content length before "VmSize:" changes
+             (typically by 1 or 2 chars) */
+          i = (i > 16) ? i -16 : 0;
+          _bft_mem_usage_proc_file_pos = i;
+        }
+        _bft_mem_usage_proc_file_init = 1;
+      }
+    }
+
+    /* Close file if unusable or if VmPeak is available */
+    if (   _bft_mem_usage_proc_file_init == -1
+        || _bft_mem_usage_proc_file_peak == 1) {
+      (void)close(_bft_mem_usage_proc_file_fd);
+      _bft_mem_usage_proc_file_fd = -1;
+    }
+  }
+
+  /* If initialization failed for some reason (proc file unavailable or does
+     or does not contain the required fields), mark method as unusable */
+  if (_bft_mem_usage_proc_file_init == 0)
+    _bft_mem_usage_proc_file_init = -1;
+
+#if defined(HAVE_MALLOC_HOOKS)
+  /* Unset tracking hooks if unneeded */
+  _bft_mem_usage_unset_hooks();
+#endif
+}
+
+/*!
+ * \brief Finalize current process memory use count depending on system.
+ */
+
+static void
+_bft_mem_usage_pr_size_end(void)
+{
+  if (_bft_mem_usage_proc_file_init != 1)
+    return;
+
+  if (_bft_mem_usage_proc_file_fd != -1) {
+    (void)close(_bft_mem_usage_proc_file_fd);
+    _bft_mem_usage_proc_file_init = 0;
+    _bft_mem_usage_proc_file_fd = -1;
+  }
+}
+
+#else  /* defined (bft_OS_Linux) && ... */
+
+#define _bft_mem_usage_pr_size_init()
+#define _bft_mem_usage_pr_size_end()
+
+#endif /* defined (bft_OS_Linux) && ... */
+
 /*============================================================================
  * Public function definitions
  *============================================================================*/
@@ -526,7 +489,7 @@ bft_mem_usage_init(void)
   if (_bft_mem_usage_global_initialized != 0)
     return;
 
-#if defined(HAVE_SBRK)
+#if defined(USE_SBRK)
 
   /*
     We use sbrk() to know the size of the heap. This is not of any use
@@ -536,12 +499,11 @@ bft_mem_usage_init(void)
 
   _bft_mem_usage_global_init_sbrk = (void *) sbrk(0);
 
-#endif /* (HAVE_SBRK) */
+#endif /* (USE_SBRK) */
 
 #if defined(HAVE_MALLOC_HOOKS)
 
-  if (_bft_mem_usage_global_track_mode != 0)
-    _bft_mem_usage_set_hooks();
+  _bft_mem_usage_set_hooks();
 
 #endif
 
@@ -576,74 +538,6 @@ bft_mem_usage_initialized(void)
   return _bft_mem_usage_global_initialized;
 }
 
-/*!
- * \brief Change memory usage tracking options for certain systems.
- *
- * On certain systems, such as GNU/Linux systems, we use hooks to the libc's
- * memory allocation functions to automatically update the maximum memory
- * allocation info (unless the BFT library was configured with malloc hooks
- * disabled or we prelink with another memory allocation library). In that
- * case, we may choose to selectively deactivate tracking with functions of
- * the bft_mem_usage_pr_size() family if we do not intend to use the
- * corresponding function of the bft_mem_usage_max_pr_size() family. This is
- * done by calling this function with the corresponding options set to 0. By
- * default (i.e. if this function is not called), full tracking is activated.
- *
- * If a tracking mode is known to be unnecessary, it is deactivated regardless
- * of the argument; for example, tracking is necessary for older Linux
- * kernels with the /proc filesystem, but not on more recent 2.6.x kernels
- * in which the "/proc/pid/status" file contains a "VmPeak" record.
- *
- * This function may be called at any time, whether before or after
- * bft_mem_usage_init(). It is not thread-safe.
- *
- * \param [in] mode mask indicating activation or deactivation of tracking
- *                  with the different available methods.
- */
-
-void
-bft_mem_usage_set_options(const int mode)
-{
-#if defined(HAVE_MALLOC_HOOKS)
-
-  _bft_mem_usage_global_track_mode = mode;
-
-  _limit_track_mode();
-
-  if (_bft_mem_usage_global_track_mode == 0)
-    _bft_mem_usage_unset_hooks();
-  else
-    _bft_mem_usage_set_hooks();
-
-#endif
-}
-
-/*!
- * \brief Return memory usage tracking options for certain systems.
- *
- * Tracking is only necessary on certain systems ;
- * see bft_mem_usage_set_options() for explanations.
- *
- * This function may be called at any time, whether before or after
- * bft_mem_usage_init().
- *
- * \param [out] mode mask indicating activation or deactivation of tracking
- *                   with the different available methods.
- */
-
-void
-bft_mem_usage_get_options(int  *const mode)
-{
-  *mode = 0;
-
-#if defined(HAVE_MALLOC_HOOKS)
-
-  if (_bft_mem_usage_global_initialized == 1)
-    *mode = _bft_mem_usage_global_track_mode;
-
-#endif
-}
-
 /*!
  * \brief Return current process memory use (in kB) depending on system.
  *
@@ -861,6 +755,33 @@ bft_mem_usage_pr_size(void)
   return sys_mem_usage;
 }
 
+#elif defined(USE_SBRK)
+
+size_t
+bft_mem_usage_pr_size(void)
+{
+  size_t alloc_size = 0;
+
+  if (_bft_mem_usage_global_initialized) {
+    void    *end_addr;
+
+    end_addr = (void *) sbrk(0);
+
+#if defined(HAVE_PTRDIFF_T)
+    alloc_size = (size_t)(  (ptrdiff_t)end_addr
+                          - (ptrdiff_t)_bft_mem_usage_global_init_sbrk) / 1024;
+#else
+    alloc_size = (end_addr - _bft_mem_usage_global_init_sbrk) / 1024;
+#endif
+
+  }
+
+  if (alloc_size > _bft_mem_usage_global_max_pr)
+    _bft_mem_usage_global_max_pr = alloc_size;
+
+  return alloc_size;
+}
+
 #elif defined(HAVE_GETRUSAGE)
 
 size_t
@@ -886,83 +807,6 @@ bft_mem_usage_pr_size(void)
 
 #endif /* bft_OS_Linux, bft_OS_OSF1, ... */
 
-/*!
- * \brief Return current measurable allocatable memory use (in kB)
- *
- * If the information is not measurable in a dependable fashion
- * or bft_mem_usage_init() has not been called, 0 is returned.
- */
-
-size_t
-bft_mem_usage_alloc_size(void)
-{
-  size_t alloc_size = 0;
-
-#if defined(USE_SBRK)
-
-  if (_bft_mem_usage_global_initialized) {
-    void    *end_addr;
-
-    end_addr = (void *) sbrk(0);
-
-#if defined(HAVE_PTRDIFF_T)
-    alloc_size = (size_t)(  (ptrdiff_t)end_addr
-                          - (ptrdiff_t)_bft_mem_usage_global_init_sbrk) / 1024;
-#else
-    alloc_size = (end_addr - _bft_mem_usage_global_init_sbrk) / 1024;
-#endif
-
-  }
-
-#endif /* (USE_SBRK) */
-
-  if (alloc_size > _bft_mem_usage_global_max_alloc)
-    _bft_mem_usage_global_max_alloc = alloc_size;
-
-  return alloc_size;
-}
-
-/*!
- * \brief Return current heap size (in kB)
- *
- * The sbrk() function is used to know the size of the heap. This is not
- * of any use to guess at allocated memory when some part of the memory may
- * be allocated with mmap(), such as with glibc on Linux.
- *
- * If the sbrk() function call is not available or bft_mem_usage_init()
- * has not been called, 0 is returned.
- */
-
-size_t
-bft_mem_usage_heap_size(void)
-{
-
-  size_t heap_size = 0;
-
-#if defined(HAVE_SBRK)
-
-  if (_bft_mem_usage_global_initialized) {
-    void    *end_addr;
-
-    end_addr = (void *) sbrk(0);
-
-#if defined(HAVE_PTRDIFF_T)
-    heap_size = (size_t)(  (ptrdiff_t)end_addr
-                         - (ptrdiff_t)_bft_mem_usage_global_init_sbrk) / 1024;
-#else
-    heap_size = (end_addr - _bft_mem_usage_global_init_sbrk) / 1024;
-#endif
-
-  }
-
-#endif /* (HAVE_SBRK) */
-
-  if (heap_size > _bft_mem_usage_global_max_heap)
-    _bft_mem_usage_global_max_heap = heap_size;
-
-  return heap_size;
-}
-
 /*
  * \brief Return maximum process memory use (in kB) depending on OS.
  *
@@ -983,46 +827,6 @@ bft_mem_usage_max_pr_size(void)
   return _bft_mem_usage_global_max_pr;
 }
 
-/*
- * \brief Return maximum measurable allocatable memory use (in kB).
- *
- * The returned value is the maximum returned by bft_mem_usage_alloc_size()
- * during the program's lifetime. With memory allocations which return
- * memory to the system (such as the GNU glibc on Linux systems),
- * this value will be correct only if allocation is tracked. This should
- * be the case if malloc hooks are used with the glibc allocation
- * functions (BFT library's default configuration/installation option),
- * but may give results lower than the true maximum in other cases.
- */
-
-size_t
-bft_mem_usage_max_alloc_size(void)
-{
-  (void) bft_mem_usage_alloc_size();
-
-  return _bft_mem_usage_global_max_alloc;
-}
-
-/*
- * \brief Return maximum heap size (in kB).
- *
- * The returned value is the maximum returned by bft_mem_usage_heap_size()
- * during the program's lifetime. With memory allocations which return
- * memory to the system (such as the GNU glibc on Linux systems),
- * this value will be correct only if allocation is tracked. This should
- * be the case if malloc hooks are used with the glibc allocation
- * functions (BFT library's default configuration/installation option),
- * but may give results lower than the true maximum in other cases.
- */
-
-size_t
-bft_mem_usage_max_heap_size(void)
-{
-  (void) bft_mem_usage_heap_size();
-
-  return _bft_mem_usage_global_max_heap;
-}
-
 /*----------------------------------------------------------------------------*/
 
 #ifdef __cplusplus
